#!/usr/bin/env python
##------------------------------------------------------------------------------
##
## create_fake_files_for_cos_cat.py 
##
##  Script intended to create copies of certain files for COS observations so
##  current catalog system can get keyword values from extra file extensions.
## 
##  Read from ingest_files_mapping (IFM) in keyword database to determine new 
##  suffix for the copied files.  The important thing here is that we use the 
##  suffixes from IFM when we make the copies because catalog will use the 
##  ifm_suffix information, too.  BTW: ipppssoot_suffix.fits
## 
##  Because some types of observations require two file copies, this idea is 
##  NOT straightforward and is impossible to automate w/o some kind of hard-coding.  
##  IF a rule is followed in IFM for entries having a suffix to be copied
##  to end ifm_suffix with "copy", and corresponding ifm_fileclass with "2"
##  then we can formulate a very specific query to get the correct
##  information.   I have assumed this rule in this code.  If IFM changes,
##  the query here should fail:  if the ifm_fileclass does not end in "2"
##  and ifm_suffix in "copy", the query done here will return too few
##  results, if more cases are added to ifm, the query will return too many
##  results.  In either case, this will cause the program to go absent as
##  either of these changes would affect ALL observations.
## 
##  Messages are written only to STDOUT, which in a pipeline is the process log 
##  file.  We do not want trailer additions because the data have already been 
##  ingested.
##
## Input:
##   Pipeline: ENV VARS: OSF_DATASET, OUTPATH, KW_DB, KW_SERVER 
##   Cmd Line: If ENV VARS are NOT defined, 4 arguments are expected
##
## History:
## yy/mm/dd PR    Who        Description
## -------- ----- ---------- ---------------------------------------------------
## 08/12/16 60896 Sherbert   Near final version
## 08/17/09 63029 Sherbert   remove the from import: improvements, I hope
## 
##------------------------------------------------------------------------------
import os
import sys
import glob
import opusutil
import ac_pkg 

## Sometimes maybe you want the message to look
## like it comes from the calling program
moduleName = os.path.basename(sys.argv[0])
## but errors should specify this script
thisModule = 'create_fake_files_for_cos_cat'

##------------------------------------------------------------------------------
##
## Method:  usage()
##
## Description: 
##          Print usage message for create_fake_files_for_cos_cat(.py)
##          Not sure what the status of this becomes if this files becomes
##          a package as opposed to a script.  Do I rename it to be more 
##          specific to the fake files creation? (def fakeFilesUsage()?)
##
## Returns: nothing
##
##------------------------------------------------------------------------------
def usage():
    usageMsg = """
            create_fake_files_for_cos_cat.py

            If no command-line arguments are provided, environment variables 
            must be defined for the 4 arguments.  Otherwise, the arguments are 
            expected to be in this order:

            <KW_SERVER> <KW_DB> <OSF_DATASET> <OUTPATH>

            <KW_SERVER>, <KW_DB> are the keyword server and database 
            <OSF_DATASET> is expected to exist in <OUTPATH>.  
            
            This script will create copies of certain COS files so Catalog can 
              read from  extra extensions.   If input is NOT COS, script exits
              quietly and successfully.

            To catalog from the SNAP2 extension:  
            - all singletons and members (exposures) should have a copy 
              of the *_spt.fits file.  
            - all associations should have a *_jnk.fits file.

            To catalog from the SCI and SNAP1 extensions:  
            - all target acqs should have copies of both 
              the *_rawacq.fits file and the *_spt.fits file.

            The names of the copies will match an ifm_suffix entry in the 
            <KW_DB> ingest_files_mapping table on <KW_SERVER>.

    """
    opusutil.PrintMsg('F',usageMsg,thisModule)
    return


##------------------------------------------------------------------------------
##
## Method: doIt
##
## Input:
##     a hash keyed on KW_SERVER, KW_DB, OSF_DATASET, and OUTPATH
##
## Description: heart of program, callable from another python script
##
## History:
## yy/mm/dd PR    Who        Description
## 09/08/07 63029 Sherbert   begin clean up of sys.exit()
## 08/12/09 63318 Sherbert   catch an exception
## 07/15/10 57271 MSwam      remove db retry parameter
##
##------------------------------------------------------------------------------
def doIt(inputVars):   

    KW_SERVER = inputVars["KW_SERVER"] 
    KW_DB     = inputVars["KW_DB"] 
    rootname  = inputVars["OSF_DATASET"]
##  dir       = inputVars["OUTPATH"]

    ## Seems a little late but is only way to get rootname in start msg.
    msg = '-- Starting  create_fake_files_for_cos_cat for '+rootname 
    opusutil.PrintMsg('I', msg, moduleName)

    ## Better to fail early if we cannot read the db
    try:
        newSuffixChoices = ac_pkg.getIFMcopySuffixes(KW_SERVER, KW_DB)
    except RuntimeError, errMsg:
        opusutil.PrintMsg('F', str(errMsg), 'ac_pkg.getIFMcopySuffixes' )
        ## An error here likely indicates a db problem
        return ac_pkg.GO_ABSENT

    ## HST Unix files can be counted on to be lower-case 
    rootname = rootname.lower()
    curDir = os.getcwd()
    debug = "Current dir is " + curDir
    opusutil.PrintMsg('D', debug )
    fileMask = rootname + "*.fits"
    debug = "Look for datasets like " + fileMask
    opusutil.PrintMsg('D', debug )
    ## Sort to assure the order jnk, rawacq, spt (want spt last) 
    files = glob.glob(fileMask)
#   print "Files are ......", files
    files = sorted(files)
#   print "Sorted files are", files
    ## Use sorted because it returns a copy of the list, l.sort() does not.  
    ## That does NOT mean I could use files.sort() in the for loop because 
    ## if you try to, you get "TypeError: 'NoneType' object is not iterable"
    debug = "Found " + str( len(files) ) + " " + fileMask
    opusutil.PrintMsg('D', debug )

    ## Easiest to determine ASNs by looking at charNine
    ## but then have to resort to filenames, therefore 
    ## consistently determine by looking at files available 
    ## for the rootname fed in?  Already cd-ed to caldir...
    try: 
        info = ac_pkg.COSfileTypes( files )
    ## Error case 
    except RuntimeError, errMsg:
        opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.COSfileTypes' )
       #msg = "Could NOT determine which files to copy!  "
       #opusutil.PrintMsg('E', msg, thisModule )
        msg = "Could this be an uncalibrated association? "
        opusutil.PrintMsg('E', msg, thisModule )
        return ac_pkg.QUIT_THIS_DATASET

    type      = info[0]
    file2copy = info[1]
    msg = rootname + " is this type: " + str(type)
    opusutil.PrintMsg('D', msg)

    ## Here is where the hard-coding comes into play.
    ## But the good thing about this is that if anyone changes IFM
    ## so that this string is no longer an ifm_fileclass, then we 
    ## will fail to find suffix... THEN WHAT HAPPENS?

    ## The rawacq is a simple copy, but all SPTs require a function call
    ## I want parallel actions subroutined (methodized?)
    if type == "ACQ":
        ## Copy rawacq file
        try:
            suffix = ac_pkg.getSuffix(newSuffixChoices, 'rawacq2') 
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.getSuffix')
            return ac_pkg.QUIT_THIS_DATASET
        msg = "ACQ1: suffix is " + suffix
        opusutil.PrintMsg('D', msg)
        try:
            ac_pkg.createNewFile( rootname, suffix, file2copy  )
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.createNewFile')
            return ac_pkg.QUIT_THIS_DATASET
    
        ## Copy acq SPT file
        try:
           suffix = ac_pkg.getSuffix(newSuffixChoices, 'sptacq2')
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.getSuffix')
            return ac_pkg.QUIT_THIS_DATASET
        acqSPT = rootname + "_spt.fits"
        msg = "ACQ2: suffix is " + suffix
        opusutil.PrintMsg('D', msg)
        try:
            ac_pkg.createNewSPT( rootname, suffix, acqSPT, "SNAP1" )
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.createNewSPT')
            return ac_pkg.QUIT_THIS_DATASET
    
    elif type == "ASN":
        ## For calibrated ASNs (file2copy has a value)
        ## Copy jnk SPT file
        try:
            suffix = ac_pkg.getSuffix(newSuffixChoices, 'sptjnk2') 
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.getSuffix')
            return ac_pkg.QUIT_THIS_DATASET
        msg = "ASN: suffix is " + suffix
        opusutil.PrintMsg('D', msg)
        try:
            ac_pkg.createNewSPT( rootname, suffix, file2copy, "SNAP2" )
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.createNewSPT')
            return ac_pkg.QUIT_THIS_DATASET
    
        ## Copy each member's SPT file
        msg = "Also copy member SPTs for " + rootname
        opusutil.PrintMsg('I', msg)
        try:
            members = ac_pkg.get_lc_ASNmembers( rootname )
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.get_lc_ASNmembers')
            return ac_pkg.QUIT_THIS_DATASET
        for member in members:
            try:
                suffix = ac_pkg.getSuffix(newSuffixChoices, 'spt2') 
            except RuntimeError, errMsg:
                opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.getSuffix')
                return ac_pkg.QUIT_THIS_DATASET
            msg = "MEM: suffix is " + suffix
            opusutil.PrintMsg('D', msg)
            memberSPT = member + "_spt.fits"
            try:
                ac_pkg.createNewSPT( member, suffix, memberSPT, "SNAP2" )
            except RuntimeError, errMsg:
                opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.createNewSPT')
                return ac_pkg.QUIT_THIS_DATASET
    
    else:
        ## Copy normal exposure's SPT file
        try:
            suffix = ac_pkg.getSuffix(newSuffixChoices, 'spt2') 
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.getSuffix')
            return ac_pkg.QUIT_THIS_DATASET
        msg = "EXP: suffix is " + suffix
        opusutil.PrintMsg('D', msg)
        try:
            ac_pkg.createNewSPT( rootname, suffix, file2copy, "SNAP2" )
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.createNewSPT')
            return ac_pkg.QUIT_THIS_DATASET

    msg = '-- Completed create_fake_files_for_cos_cat for '+rootname 
    opusutil.PrintMsg('I', msg, moduleName)
    return ac_pkg.ALL_IS_WELL


##------------------------------------------------------------------------------
##
## Main
##
## History:
## mm/dd/yy PR    Who        What          
## -------- ----- ---------- --------------------------------------------------
## 08/12/09 63318 Sherbert   catch an exception
##
##------------------------------------------------------------------------------
if __name__ == "__main__":
    ac_pkg.banner()
    ## Want this script to be runnable from command line or from pipeline
    ## or callable from another python script
    envVars = {}
    needThese = ["KW_SERVER", "KW_DB", "OSF_DATASET", "OUTPATH"]
    if (len(sys.argv)) == 1:
        opusutil.PrintMsg('D', 'Processing env vars', moduleName )
        try:
            envVars = ac_pkg.envSetUp('env', needThese)
        except RuntimeError, errMsg:
            opusutil.PrintMsg('F', str(errMsg), thisModule )
            usage()
            sys.exit(ac_pkg.GO_ABSENT)
    else:
        opusutil.PrintMsg('D', 'Processing cmd line vars', moduleName )
        envVars = ac_pkg.envSetUp('cmdLine', needThese)
    
    ## If another python script calls this one, it will need to do these, too.
    ## Verify we are dealing with COS data
    rootname  = envVars["OSF_DATASET"]
    if ac_pkg.verify_SI(rootname, "COS"):
        caldir    = envVars["OUTPATH"]

        ## Just to make sure all stretches are stretched:
        caldir = opusutil.StretchFile(caldir)
        #msg = "rootname is " + rootname + " and caldir is " + caldir
        #opusutil.PrintMsg('D', msg)

        ## Verify and cd to caldir
        try:
            ac_pkg.verify_n_cd_to_dir(caldir)
        except RuntimeError, errMsg:
            opusutil.PrintMsg('E', str(errMsg), 'ac_pkg.verify_n_cd_to_dir')
            sys.exit(ac_pkg.GO_ABSENT)

        status = doIt(envVars)
        sys.exit(status)

    else:
        opusutil.PrintMsg('E', 'Exiting. NOT a COS dataset.', thisModule )
        sys.exit(ac_pkg.QUIT_THIS_DATASET)
